#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>

/*
Changelog:
Fixed compile conflict
Fixed error with argv[1]
Added comments
*/

/* Holds widgets for g_signal_connect */
typedef struct {
	GtkWidget *file, *pass, *window;
} Widgets;

/* Creates the full filename for argv[1] */
gchar *parse_file_uri(gchar *uri) {
	gchar *filename;
	if(strstr(uri, ":")) {
		if(g_strstr_len(uri, 5, "file:")) {
			filename = g_filename_from_uri(uri, NULL, NULL);
		} else {
			return NULL;
		}
	} else {
			if(g_path_is_absolute(uri)) {
				filename = g_strdup(uri);
		} else {
			filename = g_build_filename(g_get_current_dir(), uri, NULL);
		}
	}
return filename;
}

/* Displays error messages */
static void warning_dialog(const char *title, const char *message, GtkWidget *widget) {
	GtkWidget *dialog;
	dialog = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(widget))), GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, message);
	gtk_window_set_title(GTK_WINDOW(dialog), title);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
}

/* The encrypt action */
static void encrypt_sub(GtkWidget *widget, Widgets *info) {
	/* Variables */
	gchar *filename;
	const gchar *passphrase;
	filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(info->file));
	passphrase = gtk_entry_get_text(GTK_ENTRY(info->pass));

	/* Command to be executed */
	char echo_start[100] = "echo \"";
	char echo_end[100] = "\" | ";
	char gpg_start[100] = "gpg -c --batch --passphrase-fd 0 \"";
	char gpg_end[100] = "\"";

	/* File check */
	FILE *exist;
	if((exist = fopen(filename,"r")) == NULL) {
		printf("Cannot open file %s\n", filename);
		warning_dialog("Cannot Open File", "The file cannot be found.", GTK_WIDGET(widget));
	} else {
		/* Combine strings */
		strcat(echo_start, passphrase);
		strcat(echo_start, echo_end);
		strcat(echo_start, gpg_start);
		strcat(echo_start, filename);
		strcat(echo_start, gpg_end);
		system(echo_start);
		exit(0);
	}
}

/* The decrypt action */
static void decrypt_sub(GtkWidget *widget, Widgets *info) {
	/* Variables */
	gchar *filename;
	const gchar *passphrase;
	filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(info->file));
	passphrase = gtk_entry_get_text(GTK_ENTRY(info->pass));

	/* Command to be executed */
	char echo_start[100] = "echo \"";
	char echo_end[100] = "\" | ";
	char gpg_start[100] = "gpg --batch --passphrase-fd 0 \"";
	char gpg_end[100] = "\"";

	/* File check */
	FILE *exist;
	if((exist = fopen(filename,"r")) == NULL) {
		printf("Cannot open file %s\n", filename);
		warning_dialog("Cannot Open File", "The file cannot be found.", GTK_WIDGET(widget));
	} else {
		/* Combine strings */
		strcat(echo_start, passphrase);
		strcat(echo_start, echo_end);
		strcat(echo_start, gpg_start);
		strcat(echo_start, filename);
		strcat(echo_start, gpg_end);
		system(echo_start);
		exit(0);
	}
}

/* The main code execution and display */
int main(int argc, char *argv[]) {
	/* Variables */
	GtkWidget *vbox;
	GtkWidget *button;
	GtkWidget *dialog;
	Widgets *info = (Widgets*) g_malloc(sizeof(Widgets));
	gtk_init(&argc, &argv);

	/* Window attributes */
	info->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_position(GTK_WINDOW(info->window), GTK_WIN_POS_MOUSE);
	gtk_widget_set_size_request(GTK_WIDGET(info->window), 220, 125);
	gtk_window_set_title(GTK_WINDOW(info->window), "GPG GUI");
	g_signal_connect(G_OBJECT(info->window), "destroy", G_CALLBACK(gtk_main_quit), NULL);
	g_signal_connect_swapped(G_OBJECT(info->window), "delete_event", G_CALLBACK(gtk_widget_destroy), G_OBJECT(info->window));
	gtk_container_set_border_width(GTK_CONTAINER(info->window), 3);

	/* Vbox for packing */
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(info->window), vbox);
	gtk_widget_show(vbox);

	/* File picker dialog */
	dialog = gtk_file_chooser_dialog_new("Open File", GTK_WINDOW(info->window), GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

	/* File picker button */
	info->file = gtk_file_chooser_button_new_with_dialog(dialog);
	/* Set argv[1] */
	if(argv[1] != NULL) {
		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(info->file), parse_file_uri(argv[1]));
		#ifdef WIN32
			gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(info->file), argv[1]);
		#endif
	}
	gtk_box_pack_start(GTK_BOX(vbox), info->file, TRUE, TRUE, 2);
	gtk_widget_show(info->file);

	/* Password section */
	info->pass = gtk_entry_new();
	g_signal_connect(G_OBJECT(info->pass), "activate", G_CALLBACK(decrypt_sub), (gpointer) info);
	gtk_box_pack_start(GTK_BOX(vbox), info->pass, TRUE, TRUE, 0);
	gtk_entry_set_visibility(GTK_ENTRY(info->pass), FALSE);
	gtk_widget_show(info->pass);

	/* Decrypt button */
	button = gtk_button_new_with_label("Decrypt");
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(decrypt_sub), (gpointer) info);
	gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 2);
	gtk_widget_show(button);

	/* Encrypt button */
	button = gtk_button_new_with_label("Encrypt");
	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(encrypt_sub), (gpointer) info);
	gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 2);
	gtk_widget_show(button);

	/* Show windows */
	gtk_widget_show(info->window);
	gtk_main();

	return 0;
}
